@@ -54,24 +54,26 @@ class RegionEstimator(object):
         ### Check actuals
         assert 'timestamp' in list(actuals.columns), "There is no timestamp column in actuals dataframe"
         assert 'sensor_id' in list(actuals.columns), "There is no sensor_id column in actuals dataframe"
-
-        # Check measurement columns have numeric or null data.
-        for column in list(actuals.columns):
-            if column not in ['timestamp', 'sensor_id']:
-                df_temp = actuals.loc[actuals[column].notnull()]
-                assert pd.to_numeric(df_temp[column], errors='coerce').notnull().all(), \
-                    "actuals['" + column + "'] column contains non-numeric values (null values accepted)."
-
-        # Check that each sensor ID is present in the sensors dataframe index.
-        allowed_sensors = sensors.index.values
-        print(allowed_sensors)
-        #assert  , "Each sensor ID must match a sensor_id in sensors."
-
         assert len(list(actuals.columns)) > 2, "There are no measurement value columns in the actuals dataframe."
 
+        # Check measurement columns have either numeric or null data
+        for column in list(actuals.columns):
+            if column not in ['timestamp', 'sensor_id']:
+                # Check measurement does not contain numeric (nulls are OK)
+                df_temp = actuals.loc[actuals[column].notnull()]
+                assert pd.to_numeric(df_temp[column], errors='coerce').notnull().all(), \
+                    "actuals['" + column + "'] column contains non-numeric values (null values are accepted)."
 
 
 
+        # Check that each sensor_id value is present in the sensors dataframe index.
+        # ... So sensor_id values must be a subset of allowed sensors
+        error_sensors = set(actuals['sensor_id'].unique()) - set(sensors.index.values)
+        assert len(error_sensors) == 0, \
+            "Each sensor ID must match a sensor_id in sensors. Error sensor IDs: " + str(error_sensors)
+
+
+        ### Convert to geo dataframe
         try:
             gdf_sensors = gpd.GeoDataFrame(data=sensors,
                                            geometry=gpd.points_from_xy(sensors.longitude, sensors.latitude))
@@ -85,15 +87,19 @@ class RegionEstimator(object):
         except Exception as err:
             raise ValueError('Error converting regions DataFrame to a GeoDataFrame: ' + str(err))
 
-        self.sensors = gdf_sensors
-        self.regions = gdf_regions
 
-        # Make all non integer values Null in measurement fields
+
         #   Make sure value columns at the end of column list
         cols = actuals.columns.tolist()
         cols.insert(0, cols.pop(cols.index('sensor_id')))
         cols.insert(0, cols.pop(cols.index('timestamp')))
-        actuals[cols[2:]] = actuals[cols[2:]].apply(pd.to_numeric, errors='coerce')
+
+        # Make all non integer values Null in measurement fields
+        # No longer required, as covered in 'check actuals' assertations above
+        #actuals[cols[2:]] = actuals[cols[2:]].apply(pd.to_numeric, errors='coerce')
+
+        self.sensors = gdf_sensors
+        self.regions = gdf_regions
         self.actuals = actuals
 
         self.__get_all_region_neighbours()
